#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>

#include "lcd.h"

#define SENSOR1   PD5
#define SENSOR2   PD4

#define Sensor_DDR    DDRD
#define Sensor_PORT   PORTD
#define Sensor_PIN    PIND

//#define BUZZER    PD4

// Pinii pentru banda de leduri RGB
#define LED_RED       PD0
#define LED_GREEN     PD1
#define LED_BLUE      PD2

#define Led_DDR   DDRD
#define Led_PORT  PORTD
#define Led_PIN   PIND

#define START   PD6
#define LED_TEST  PD7

#define WINNER  3

volatile int score_red;
volatile int score_blue;

void print_score(int score_red, int score_blue)
{
		char buf[20];
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		LCD_writeInstr(LCD_INSTR_DDRAM);
    LCD_print("   RED  |  BLUE ");
		LCD_writeInstr(LCD_INSTR_nextLine);
		sprintf(buf, "    %d   |   %d   ", score_red, score_blue);
		LCD_print(buf);
}
static void set_sensor(uint8_t sensor) {
  /* Senzorul este pe portul 'sensor' */
 	/* Setez 'sensor' ca intrare */
  /* Activez rezistenta de pull-up pentru Senzor*/
 	Sensor_DDR &= ~(1 << sensor);
  Sensor_PORT |= (1 << sensor);
}

static void set_start(uint8_t button) {
 	DDRD &= ~(1 << button);
  PORTD |= (1 << button);
}

static void set_rgb_led(uint8_t led) {
   Led_DDR |= (1 << led);
}

// static void set_buzzer(uint8_t buzzer) {
//   /* Activare iesire buzzer */
//   DDRD |= ( 1 << buzzer);
//   /* buzzer oprit */
//   PORTD |= ( 1 << buzzer);
// }

// static void buzzer_on(uint8_t buzzer) {
//   PORTD &= ~(1 << buzzer);
// }
//
// static void buzzer_off(uint8_t buzzer) {
//   PORTD |= (1 << buzzer);
// }

static void led_on(uint8_t led) {
  Led_PORT |= (1 << led);
}

static void led_off(uint8_t led) {
  Led_PORT &= ~(1 << led);
}

void print_gameover(char* winner) {
	char buf[20];
	LCD_writeInstr(LCD_INSTR_clearDisplay);
	LCD_writeInstr(LCD_INSTR_DDRAM);
	LCD_print("  GAME OVER  ");
	LCD_writeInstr(LCD_INSTR_nextLine);
	sprintf(buf, "    %s WINS   ", winner);
	LCD_print(buf);

}

ISR(PCINT3_vect) {
    if ((Sensor_PIN & (1 << SENSOR2)) == 0) {
      /* rosu */
        score_red++;
        print_score(score_red,score_blue);
        led_off(LED_BLUE);
        led_on(LED_RED);
      //  buzzer_on(BUZZER);
        _delay_ms(3000);
        //buzzer_off(BUZZER);
        led_off(LED_RED);
        if (score_red == WINNER) {
          print_gameover("RED");
          score_red = 0;
          score_blue = 0;
        }


    }
    if ((Sensor_PIN & (1 << SENSOR1)) == 0) {
        /* albastru */
        score_blue++;
        print_score(score_red,score_blue);
        led_off(LED_RED);
        led_on(LED_BLUE);
        //buzzer_on(BUZZER);
        _delay_ms(3000);
        //buzzer_off(BUZZER);
        led_off(LED_BLUE);
        if (score_blue == WINNER) {
          print_gameover("BLUE");
          score_red = 0;
          score_blue = 0;
        }

    }

}
/* Initializeaza intrarile si iesirile */
void IO_init() {
    set_sensor(SENSOR1);
    set_sensor(SENSOR2);
    //set_buzzer(BUZZER);

    /* Banda leduri RGB */
    set_rgb_led(LED_RED);
    set_rgb_led(LED_GREEN);
    set_rgb_led(LED_BLUE);

    /* Activez PIN CHANGE INTERRUPT ENABLE
     * pt PCINT28 (Senzor1) si PCINT29(Senzor2)
     */
    PCICR |= (1 << PCIE3);

    /* Activez intreruperea pe Senzori */
    PCMSK3 |= (1 << PCINT29) | (1 << PCINT28);
}


int main(void) {

    sei();
    IO_init();
    LCD_init();
    print_score(score_red,score_blue);

    while (1) {
    }
    return 0;
}
